BeginPackage["HEPMath`Symbolics`"]

GetDefinition::usage = "GetDefinition[sym] returns the full definition of the
symbol sym in the form {prop1->val1, prop2->val2, ...} where prop1, prop2, ...
are properties like DownValues, OwnValues etc."

SetDefinition::usage = "SetDefinition[sym, def] sets the definition of symbol
sym to def. def must be an expression of the form
 {prop1->val1, prop2->val2, ...} where prop1, prop2, ... are properties like
DownValues, OwnValues etc. See also GetDefinition."

Rename::usage = "Rename[sym, newsym] copies the definition of sym to newsym and
removes sym. If the option UpdateContexts is given, sym is replaced by newsym
in the definitions of all symbols in the specified contexts. If the option
Keep is set to True the symbol sym is not removed."

UpdateContexts::usage = "UpdateContexts is an option of Rename. Allowed values
are a list of contexts or context patterns or a single context or context
pattern. Defaults to {}"

Keep::usage = "Keep is a boolean option of Rename. If set to True, the renamed
symbol is not removed from the kernel."

Begin["`Private`"]

SetAttributes[GetDefinition, HoldFirst]

GetDefinition[sym_] := (# -> #[sym])& /@
  {Attributes, UpValues, DownValues, SubValues,
   NValues, FormatValues, Messages, Options, OwnValues}


SetAttributes[SetDefinition, HoldFirst]

SetDefinition[sym_Symbol, def_] := (
  ClearAll[sym];
  Scan[Set[#[sym], (# /. def)]&, First /@ def];
)

SetDefinition[sym_String, def_] := (
  ClearAll[sym];
  SetDefinition[Evaluate[Symbol[sym]], def]
)


renameInDef[sym_, newsym_][name_] := Block[{def},
  def=GetDefinition@@ToHeldExpression[name];
  If[!FreeQ[def, sym],
    ClearAll[name];
    SetDefinition[name, def /. sym -> newsym]];
]

renameInContext[sym_, newsym_][cont_] :=
  renameInDef[sym, newsym] /@ Names[cont<>"*"]


SetAttributes[Rename, HoldAll]

Options[Rename] = {UpdateContexts->{}, Keep->False}

Rename[sym_, newsym_, OptionsPattern[]] := Block[
  {def=GetDefinition[sym],
   conts=OptionValue[UpdateContexts]},

  conts = If[Head[conts]===List, Flatten[Contexts /@ conts], Contexts[conts]];
  ClearAll[sym];
  ClearAll[newsym];
  newsym;
  Scan[renameInContext[sym, newsym], conts];
  SetDefinition[newsym, def /. sym -> newsym];
  If[OptionValue[Keep], SetDefinition[sym, def], Remove[sym]];
]



End[]

EndPackage[]

