PrependTo[$Path, "../.."]

Needs["HEPMath`"]

Needs["HEPMath`Compile`"]

ltresults = Get["ltresults.mo"]

approxEqual[a_, b_] := Abs[a-b] < Max[10.0^(-14)*Max[Abs[a],Abs[b]], 10.0^(-14)]

DeclareLorentzVectors[p,q,e1,e2]


testHEPPrepare[] := Block[{tests, mu, m, expr, testExpr},
  mu = 172.0;

  expr = {
    PaVe[1,mu,0][0][Sqr[p]]*Den[p, m] +
      PaVe[2,mu,0][1][Sqr[p], SDot[p,q], SDot[p,q]],
    PaVe[2,mu,0][0][Sqr[p], 0, 0],
    PaVe[2,mu,0][0][Sqr[p], 10.0^(-14), 0],
    PaVe[2,mu,1][0][Sqr[p], 0, 0],
    PaVe[2,mu,2][0][Sqr[p], 0, 0]
  };

  testExpr[ex_, res_] := Block[{cex=ex},
    cex = HEPCompile[{{p, _Real, 1}, {q, _Real, 1}, {m, _Real}}, HEPPrepare[cex],
                     CompilationOptions->{"ExpressionOptimization"->True,
                                          "InlineCompiledFunctions"->True,
                                          "InlineExternalDefinitions"->True}];
    cex = cex[{400., 0., 0., 0.}, {25., 0., 0., 0.}, 15.0];
    approxEqual[cex, res]
  ];

  tests = MapThread[testExpr, {expr, Take[ltresults, Length[expr]]}];
  And @@ tests
]

testHEPCompile[] := Block[
  {tests={}, p={10., 0.3, 0.6, 1.0}, e1, e2, cex},
  cex = HEPCompile[{{sg1, _Integer}, {sg2, _Integer}},
    Block[{e1=Pol[p][sg1], e2=CC[Pol[p]][sg2]}, HEPPrepare[SDot[e1, e2]]],
    CompilationOptions->{"ExpressionOptimization"->True,
                         "InlineCompiledFunctions"->True,
                         "InlineExternalDefinitions"->True}];
  AppendTo[tests, approxEqual[cex[-1,-1], -1.0]];
  AppendTo[tests, approxEqual[cex[-1,+1],  0.0]];
  AppendTo[tests, approxEqual[cex[+1,-1],  0.0]];
  AppendTo[tests, approxEqual[cex[+1,+1], -1.0]];

  And @@ tests  
]

WriteString[$Output, "testHEPPrepare: "<>If[testHEPPrepare[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPCompile: "<>If[testHEPCompile[],"PASS\n","FAIL\n"]];

testHEPCompile[];

