PrependTo[$Path, "../.."]

Needs["HEPMath`"]

Attributes[quietCheck] = {HoldAll};
Attributes[quietCheckAbort] = {HoldAll};

quietCheck[ex_, msg_] :=
  CheckAbort[Quiet[Check[ex; False, True, msg], msg], False];
quietCheckAbort[ex_, msg_] :=
  Quiet[Check[CheckAbort[ex; False, True]; False, True, msg], msg]



testScope[] := Block[{Idx1, Idx2, t, tests={}},
  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareHEPTensor[t, {Idx1, Idx2}];
    DeclareLinear[f];
  ];

  AppendTo[tests, IndexTypeQ[Idx1] =!= True];
  AppendTo[tests, IndexTypeQ[Idx2] =!= True];
  AppendTo[tests, HEPTensorSymbolQ[t] =!= True];
  AppendTo[tests, HEPTensorSignature[t] === {}];
  AppendTo[tests, LinearQ[f] =!= True];

  And@@tests
];

testHEPTensorDeclarations[] := Block[
  {Idx1, Idx2, t, tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareHEPTensor[t, {Idx1, Idx2}];
    DeclareLinear[f];

    AppendTo[tests,
      quietCheck[DeclareIndexType[Idx1, 3], {DeclareIndexType::redecl}]];
    AppendTo[tests,
      quietCheckAbort[DeclareIndexType[t, 3], {DeclareIndexType::conflict}]];

    AppendTo[tests,
      quietCheck[DeclareHEPTensor[t, {Idx1, Idx2}], {DeclareHEPTensor::redecl}]];
    AppendTo[tests,
      quietCheckAbort[DeclareHEPTensor[t, {Idx1}], {DeclareHEPTensor::conflict}]];
    AppendTo[tests,
      quietCheckAbort[DeclareHEPTensor[t, {Idx2, Idx1}],
                      {DeclareHEPTensor::conflict}]];
    AppendTo[tests,
      quietCheckAbort[DeclareHEPTensor[t, {plus, Idx1}],
                      {DeclareHEPTensor::invidx}]];
    AppendTo[tests,
      quietCheckAbort[DeclareHEPTensor[Idx1, {Idx1, Idx2}],
                      {DeclareHEPTensor::conflict}]];

    AppendTo[tests, LinearQ[f] === True];
    
  And@@tests
  ]
];

testHEPTensorSignature[] := Block[
  {Idx1, Idx2, Idx3,
   t1, t2, t3, t11, t22, t33, t112, t122, t123,
   u1, u2, u3, u11, u22, u33, u112, u122, u123,
   i1, j1, k1, i2, j2, k2, i3, j3, k3,
   a, b, c, d,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareIndexType[Idx3, 4];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t2, u2}, {Idx2}];
    DeclareHEPTensor[{t3, u3}, {Idx3}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];
    DeclareHEPTensor[{t22, u22}, {Idx2, Idx2}];
    DeclareHEPTensor[{t33, u33}, {Idx3, Idx3}];
    DeclareHEPTensor[{t112, u112}, {Idx1, Idx1, Idx2}];
    DeclareHEPTensor[{t122, u122}, {Idx1, Idx2, Idx2}];
    DeclareHEPTensor[{t123, u123}, {Idx1, Idx2, Idx3}];

    AppendTo[tests,
      HEPTensorSignature[a t123 + b u123] === {Idx1, Idx2, Idx3}];
    AppendTo[tests,
      quietCheckAbort[HEPTensorSignature[a t123 + b u112],
        HEPTensorSignature::conflict]];
    AppendTo[tests,
      quietCheckAbort[HEPTensorSignature[a t11 t22],
        HEPTensorSignature::conflict]];
    AppendTo[tests,
      HEPTensorSignature[a t11[i1,j1] u123] === {Idx1, Idx2, Idx3}];
    AppendTo[tests,
      HEPTensorSignature[a t11[i1,j1] ** u123[i1]] === {Idx2, Idx3}];
    AppendTo[tests,
      HEPTensorSignature[t1.t2.t3] === {Idx1, Idx2, Idx3}];
    AppendTo[tests,
      HEPTensorSignature[t11.u11] === {Idx1, Idx1}];
    AppendTo[tests,
      HEPTensorSignature[t3.t11.u33.u11] === {Idx1, Idx3, Idx1}];
    AppendTo[tests,
      HEPTensorSignature[t1.t2.t3.t11.t22.t123[i1]] === {Idx1}];
    AppendTo[tests,
      quietCheckAbort[HEPTensorSignature[t123[i1]^2],
        HEPTensorSignature::conflict]];
    AppendTo[tests,
      HEPTensorSignature[a] === {}];
  ];

  And@@tests
];


testIndices[] := Block[
  {Idx1, Idx2, Idx3,
   t1, t2, t3, t11, t22, t33, t112, t122, t123,
   u1, u2, u3, u11, u22, u33, u112, u122, u123,
   i1, j1, k1, i2, j2, k2, i3, j3, k3,
   a, b, c, d,
   f,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareIndexType[Idx3, 4];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t2, u2}, {Idx2}];
    DeclareHEPTensor[{t3, u3}, {Idx3}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];
    DeclareHEPTensor[{t22, u22}, {Idx2, Idx2}];
    DeclareHEPTensor[{t33, u33}, {Idx3, Idx3}];
    DeclareHEPTensor[{t112, u112}, {Idx1, Idx1, Idx2}];
    DeclareHEPTensor[{t122, u122}, {Idx1, Idx2, Idx2}];
    DeclareHEPTensor[{t123, u123}, {Idx1, Idx2, Idx3}];
    DeclareLinear[f];

    AppendTo[tests,
      Indices[t123[i1,i2,i3]] === {{i1->Idx1, i2->Idx2, i3->Idx3},{}}];
    AppendTo[tests,
      Indices[t123[i3,i2,i1]] === {{i1->Idx3, i2->Idx2, i3->Idx1},{}}];
    AppendTo[tests,
      Indices[t123[i1,i2]] === {{i1->Idx1, i2->Idx2},{}}];
    AppendTo[tests,
      Indices[t112[i1,i1,i2]] === {{i2->Idx2},{i1->Idx1}}];
    AppendTo[tests,
      quietCheckAbort[Indices[t112[i1,i1,i2,i3]],
        {HEPTensorSignature::invidx}]];
    AppendTo[tests,
      Indices[t112[i1,j1,i2] u11[i1, j1]] ===
        {{i2->Idx2},{i1->Idx1, j1->Idx1}}];
    AppendTo[tests,
      Indices[t112[i1,i1,i2] u33[i1, i1]] ===
        {{i2->Idx2},{i1->Idx1, i1->Idx3}}];
    AppendTo[tests,
      quietCheckAbort[Indices[t11[i1,i1] u11[i1,i1]], {Indices::mult}]];
    AppendTo[tests,
      Indices[a t11[i1,j1] + b u11[i1,j1] + c t1[i1] u1[j1]] ===
        {{i1->Idx1,j1->Idx1},{}}];
    AppendTo[tests,
      quietCheckAbort[Indices[a t11[i1,j1] + b u11[i1,j1] + c t112[i1,j1]],
        {HEPTensorSignature::conflict}]];
    AppendTo[tests,
      Indices[t112[i1,j1,i2]^2] === {{},{i1->Idx1, i2->Idx2, j1->Idx1}}];
    AppendTo[tests,
      quietCheckAbort[Indices[t112[i1,i1,i2]^2], {Indices::mult}]];
    AppendTo[tests,
      quietCheckAbort[Indices[t112[i1,j1,i2]^3], {Indices::mult}]];
    AppendTo[tests,
      Indices[(t1 + u1)[i1]] === {{i1->Idx1},{}}];

    AppendTo[tests,
      Indices[f[t1[i1],t11[i1,j1]]] === {{j1->Idx1},{i1->Idx1}}];      
    AppendTo[tests,
      Indices[f[t11[i1,j1]]] === {{i1->Idx1, j1->Idx1},{}}];
  ];

  And@@tests
];


testExplicitIndices[] := Block[
  {Idx1, Idx2, Idx3,
   t1, t2, t3, t11, t22, t33, t112, t122, t123,
   u1, u2, u3, u11, u22, u33, u112, u122, u123,
   i1, j1, k1, i2, j2, k2, i3, j3, k3,
   a, b, c, d,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareIndexType[Idx3, 4];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t2, u2}, {Idx2}];
    DeclareHEPTensor[{t3, u3}, {Idx3}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];
    DeclareHEPTensor[{t22, u22}, {Idx2, Idx2}];
    DeclareHEPTensor[{t33, u33}, {Idx3, Idx3}];
    DeclareHEPTensor[{t112, u112}, {Idx1, Idx1, Idx2}];
    DeclareHEPTensor[{t122, u122}, {Idx1, Idx2, Idx2}];
    DeclareHEPTensor[{t123, u123}, {Idx1, Idx2, Idx3}];

    AppendTo[tests,
      MatchQ[ExplicitIndices[(t11.u11)[i,j]], t11[i,a_] u11[a_, j]]];
    AppendTo[tests,
      MatchQ[ExplicitIndices[t1.u1], t1[a_] u1[a_]]];
    AppendTo[tests,
      MatchQ[ExplicitIndices[(t1.t2.t3.t11.t22.t123[i])[j]],
        t1[a1_] t2[a2_] t3[a3_] t11[a1_, j] t22[a2_,b2_] t123[i,b2_,a3_]]];
    AppendTo[tests, quietCheckAbort[ExplicitIndices[t1.t11],
                                    {ExplicitIndices::suppidx}]];
    AppendTo[tests,
      MatchQ[ExplicitIndices[(a t1 + b u1).t11[i]],
        (a t1 + b u1)[k_] t11[i, k_]]];
    AppendTo[tests,
      MatchQ[ExplicitIndices[a + t1.u1], a + t1[k_] u1[k_]]];
  ];

  And@@tests
];


testHEPExpand[] := Block[
  {Idx1, Idx2, Idx3,
   t1, t2, t3, t11, t22, t33, t112, t122, t123,
   u1, u2, u3, u11, u22, u33, u112, u122, u123,
   i1, j1, k1, i2, j2, k2, i3, j3, k3,
   a, b, c, d, f,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareIndexType[Idx3, 4];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t2, u2}, {Idx2}];
    DeclareHEPTensor[{t3, u3}, {Idx3}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];
    DeclareHEPTensor[{t22, u22}, {Idx2, Idx2}];
    DeclareHEPTensor[{t33, u33}, {Idx3, Idx3}];
    DeclareHEPTensor[{t112, u112}, {Idx1, Idx1, Idx2}];
    DeclareHEPTensor[{t122, u122}, {Idx1, Idx2, Idx2}];
    DeclareHEPTensor[{t123, u123}, {Idx1, Idx2, Idx3}];
    DeclareHEPTensor[f[_], {Idx1}]
    DeclareLinear[f]

    AppendTo[tests,
      HEPExpand[((a (t1 + u1)).((b + c) t11))[i]] ===
        a (b + c) (t1 . t11)[i] + a (b + c) (u1 . t11)[i]];
    AppendTo[tests,
      HEPExpand[((a (t1 + u1)).((b + c) t11))[i], _] ===
        a b (t1 . t11)[i] + a c (t1 . t11)[i] + a b (u1 . t11)[i] +
        a c (u1 . t11)[i]];
    AppendTo[tests,
      HEPExpand[((a t1).((u2.u22[k]) t11 + c u11).(d u11))[i]] ===
      a d u2 . u22[k] (t1 . t11 . u11)[i] + a c d (t1 . u11 . u11)[i]];
    AppendTo[tests,
      HEPExpand[t1.(a t11 + b (t112 + u112).t2).u1] ===
      a t1 . t11 . u1 + b t1 . t112 . t2 . u1 + b t1 . u112 . t2 . u1];
    AppendTo[tests,
      PullScalars[((a (t1 + u1)).((b + c) t11))[i]] ===
      a (b + c) ((t1 + u1) . t11)[i]];
    AppendTo[tests,
      HEPExpand[f[t1 + u1].f[t2 - u2]] ===
      f[t1].f[t2] + f[u1].f[t2] - f[t1].f[u2] - f[u1].f[u2]];
    AppendTo[tests,
      HEPExpand[f[t1 + u1].f[t2 - u2], u2] ===
      f[t1 + u1].f[t2] - f[t1 + u1].f[u2]];

    AppendTo[tests,
      HEPExpand[HEPIntegral[Sqr[(1-a) b t1] + Sqr[a b t1], {a, 0, 1}]] ===
        b^2*HEPIntegral[(1 - a)^2, {a, 0, 1}]*Sqr[t1] +
        b^2*HEPIntegral[a^2, {a, 0, 1}]*Sqr[t1]];
    AppendTo[tests,
      HEPExpand[HEPDerivative[a c t1[j] t1[j] + b d t1[j] t1[j], t1[i]]] ===
        a*c*HEPDerivative[t1[j]^2, p[i]] + b*d*HEPDerivative[t1[j]^2, t1[i]]];
  ];

  And@@tests
];


testHEPContract[] := Block[
  {Idx1, id1, t1, t11, u1, u11, i1, j1, k1, a, b, c, d, tests={}},

  Scope[
    DeclareIndexType[Idx1, 4, id1];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];

    AppendTo[tests, HEPContract[(a t1[i1] + b u1[i1]) id1[i1, j1]] ===
      a t1[j1] + b u1[j1]];
    AppendTo[tests, HEPContract[(a t1[i1] + b u1[i1]) id1[j1, i1]] ===
      a t1[j1] + b u1[j1]];
    AppendTo[tests, HEPContract[(a t1[k1] + b u1[k1]) id1[j1, i1]] ===
      (a t1[k1] + b u1[k1]) id1[j1, i1]];
    AppendTo[tests, HEPContract[id1[j1, j1]] === 4];
    AppendTo[tests, HEPContract[id1[i1, j1] id1[j1, i1]] === 4];
    AppendTo[tests, HEPContract[id1[i1, j1] id1[i1, j1]] === 4];
  ];

  And@@tests
];


testHEPCollect[] := Block[
  {Idx1, p, q, k, l, a, b, c, d, e, mu, nu, expr, expr2, tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareHEPTensor[{p, q, k, l}, {Idx1}];
    DeclareHEPTensor[{eta1, eta2}, {Idx1, Idx1}];

    expr = SDot[k,p] m1 + b SDot[k,p] SDot[l,q] +
           c SDot[k,q] SDot[l,p] - SDot[k,q] m2 +
           d SDot[k,q] SDot[k,p] + e SDot[k,q] SDot[k,p];

    expr2 = p[mu] q[mu] + p[nu[1]] k[nu[1]] +
      p[mu] p[nu[1]] eta1[mu, nu[1]] - p[mu[1]] p[nu] eta2[mu[1], nu];

    AppendTo[tests, First[HEPCollectListed[expr, k]] ===
      {SDot[k, p], SDot[k, q], SDot[k, p]*SDot[k, q]}];
    AppendTo[tests, HEPCollect[expr, k] ===
      (d + e)*SDot[k, p]*SDot[k, q] + SDot[k, q]*(-m2 + c*SDot[l, p]) +
      SDot[k, p]*(m1 + b*SDot[l, q])];

    AppendTo[tests, MatchQ[
      HEPCollect[expr2, p, CombineTensorStructures->True],
      p[mu_] (k[mu_] + q[mu_]) +
      p[nu_] p[ro_] (eta1[nu_, ro_] -eta2[ro_, nu_])]];
  ];

  And@@tests
];


testCC[] := Block[
  {Idx1, Idx2, Idx3,
   t1, t2, t3, t11, t22, t33, t112, t122, t123,
   u1, u2, u3, u11, u22, u33, u112, u122, u123,
   i1, j1, k1, i2, j2, k2, i3, j3, k3,
   a, b, c, d,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareIndexType[Idx3, 4];
    DeclareHEPTensor[{t1, u1}, {Idx1}];
    DeclareHEPTensor[{t2, u2}, {Idx2}];
    DeclareHEPTensor[{t3, u3}, {Idx3}];
    DeclareHEPTensor[{t11, u11}, {Idx1, Idx1}];
    DeclareHEPTensor[{t22, u22}, {Idx2, Idx2}];
    DeclareHEPTensor[{t33, u33}, {Idx3, Idx3}];
    DeclareHEPTensor[{t112, u112}, {Idx1, Idx1, Idx2}];
    DeclareHEPTensor[{t122, u122}, {Idx1, Idx2, Idx2}];
    DeclareHEPTensor[{t123, u123}, {Idx1, Idx2, Idx3}];

    AppendTo[tests,
      CC[(2 a + I b) (t11.(t11 + u11))[i,j]] ===
        (2 CC[a] - I CC[b]) (CC[t11] . (CC[t11] + CC[u11]))[i, j]];
    AppendTo[tests,
      HEPTensorSignature[CC[t11.(a t11 + b u11)]] === {Idx1, Idx1}];
  ];

  And@@tests
];


testFunctions[] := Block[
  {Idx1, Idx2, t1, t11, u1, i1, k1, j2, a, b,
   tests={}},

  Scope[
    DeclareIndexType[Idx1, 4];
    DeclareIndexType[Idx2, 4];
    DeclareHEPTensor[u1, {Idx1}];
    DeclareHEPTensor[t2, {Idx2}];
    DeclareHEPTensor[t11, {Idx1, Idx1}];
    DeclareLinear[{f,g}];
    DeclareRealHolomorphic[g];

    AppendTo[tests, Indices[f[t11[i1,k1]u1[k1],t2[j2]]] ===
      {{i1 -> Idx1, j2 -> Idx2}, {k1 -> Idx1}}];
    AppendTo[tests, Indices[CC[f[t11[i1,k1]u1[k1],t2[j2]]]] ===
      {{i1 -> Idx1, j2 -> Idx2}, {k1 -> Idx1}}];
    AppendTo[tests, CC[g[a,b]] === g[CC[a], CC[b]]];
  ];

  And@@tests
];



WriteString[$Output, "testScope:                 "<>If[testScope[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPTensorDeclarations: "<>If[testHEPTensorDeclarations[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPTensorSignature:    "<>If[testHEPTensorSignature[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testIndices:               "<>If[testIndices[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testExplicitIndices:       "<>If[testExplicitIndices[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPExpand:             "<>If[testHEPExpand[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPContract:           "<>If[testHEPContract[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testHEPCollect:            "<>If[testHEPCollect[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testCC:                    "<>If[testCC[],"PASS\n","FAIL\n"]];
WriteString[$Output, "testFunctions:             "<>If[testFunctions[],"PASS\n","FAIL\n"]];

Exit[];


